@@ -41,6 +41,9 @@
 // "type" is either 't' for text or 'k' for keyword.  This method is a lot
 // easier to use, but requires that Beagle have inotify support enabled to
 // work.
+//
+// Any text property with the name beagle:inuri would be used for inuri: queries.
+//
 
 using System;
 using System.Collections;
@@ -53,6 +56,7 @@ using Beagle.Util;
 namespace Beagle.Daemon.IndexingServiceQueryable {
 
 	[QueryableFlavor (Name="IndexingService", Domain=QueryDomain.Local, RequireInotify=false)]
+	[PropertyKeywordMapping (Keyword="host", PropertyName="fixme:host", IsKeyword=false, Description="The host of this entitiy.")]
 	public class IndexingServiceQueryable : LuceneQueryable {
 
 		public IndexingServiceQueryable () : base ("IndexingServiceIndex")
@@ -129,6 +133,7 @@ namespace Beagle.Daemon.IndexingServiceQueryable {
 			indexable.Timestamp = data_file.LastWriteTimeUtc;
 			indexable.ContentUri = UriFu.PathToFileUri (data_file.FullName);
 			indexable.DeleteContent = true;
+			indexable.AddProperty( Property.New("fixme:host",uri.Host));
 
 			// Second line is the hit type
 			line = reader.ReadLine ();
@@ -240,40 +245,91 @@ namespace Beagle.Daemon.IndexingServiceQueryable {
 			ThisScheduler.Add (task);
 		}
 
-		protected override void PostAddHook (Indexable indexable, IndexerAddedReceipt receipt)
+		protected override Uri PostAddHook (Indexable indexable, IndexerAddedReceipt receipt)
 		{
 			FileInfo meta_file = indexable.LocalState ["MetaFile"] as FileInfo;
 			if (meta_file == null)
-				return;
+				return indexable.Uri;
 
 			meta_file.Delete ();
 
 			lock (pending_files)
 				pending_files.Remove (indexable.ContentUri.LocalPath);
+
+			return indexable.Uri;
 		}
 
 		private class IndexableGenerator : IIndexableGenerator {
-			private IEnumerator to_add_enumerator;
+			private IEnumerator to_add_enumerator = null, to_remove_uris_enumerator = null;
 			private int count = -1, done_count = 0;
+			// FIXME: Unused. Use this to store the submitter queryable in the localstate
+			// of the indexables and receive PostHooks() from LuceneQueryable
+			private LuceneQueryable submitter_queryable = null;
 
 			public IndexableGenerator (IEnumerable to_add)
 			{
 				this.to_add_enumerator = to_add.GetEnumerator ();
 			}
 
-			public IndexableGenerator (ICollection to_add) : this ((IEnumerable) to_add)
+			public IndexableGenerator (ICollection to_add) : this (to_add, null, null)
 			{
-				this.count = to_add.Count;
+			}
+
+			public IndexableGenerator (ICollection to_add, ICollection to_remove_uris) :
+				this (to_add, to_remove_uris, null)
+			{
+			}
+
+			public IndexableGenerator (ICollection to_add, ICollection to_remove_uris, LuceneQueryable submitter_queryable)
+			{
+				this.count = 0;
+				this.submitter_queryable = submitter_queryable;
+
+				if (to_add != null) {
+					this.to_add_enumerator = to_add.GetEnumerator ();
+					this.count += to_add.Count;
+				}
+
+				if (to_remove_uris != null) {
+					this.to_remove_uris_enumerator = to_remove_uris.GetEnumerator ();
+					this.count += to_remove_uris.Count;
+				}
+			}
+
+			public LuceneQueryable SubmitterQueryable {
+				set { submitter_queryable = value; }
 			}
 
 			public Indexable GetNextIndexable ()
 			{
+				if (to_remove_uris_enumerator != null) {
+					Uri uri_to_remove = to_remove_uris_enumerator.Current as Uri;
+					if (uri_to_remove == null)
+						return null;
+
+					return new Indexable (IndexableType.Remove, uri_to_remove);
+				}
+
+				if (to_add_enumerator == null)
+					return null;
+
 				return to_add_enumerator.Current as Indexable;
 			}
-			
+
 			public bool HasNextIndexable ()
 			{
 				++done_count;
+
+				if (to_remove_uris_enumerator != null) {
+					if (to_remove_uris_enumerator.MoveNext ())
+						return true;
+					else
+						to_remove_uris_enumerator = null;
+				}
+
+				if (to_add_enumerator == null)
+					return false;
+
 				return to_add_enumerator.MoveNext ();
 			}
 
@@ -294,19 +350,38 @@ namespace Beagle.Daemon.IndexingServiceQueryable {
 		{
 			IndexingServiceRequest isr = (IndexingServiceRequest) msg;
 
-			foreach (Uri uri in isr.ToRemove) {
-				Log.Debug ("IndexingService: Removing {0}", uri);
-				Scheduler.Task task = NewRemoveTask (uri);
-				ThisScheduler.Add (task);
+			LuceneQueryable backend = this;
+
+			if (isr.Source != null) {
+				Queryable target = QueryDriver.GetQueryable (isr.Source);
+
+				if (target == null) {
+					string err = String.Format ("Unable to find backend matching '{0}'", isr.Source);
+
+					Log.Error (err);
+					return new ErrorResponse (err);
+				}
+
+				if (! (target.IQueryable is LuceneQueryable)) {
+					string err = String.Format ("Backend '{0}' is not an indexed backend", isr.Source);
+
+					Log.Error (err);
+					return new ErrorResponse (err);
+				}
+
+				backend = (LuceneQueryable) target.IQueryable;
+				Log.Debug ("Found backend for IndexingServiceRequest: {0}", backend.IndexName);
 			}
 
 			// FIXME: There should be a way for the request to control the
 			// scheduler priority of the task.
 
-			if (isr.ToAdd.Count > 0) {
-				Log.Debug ("IndexingService: Adding {0} indexables.", isr.ToAdd.Count);
-				IIndexableGenerator ind_gen = new IndexableGenerator (isr.ToAdd);
-				Scheduler.Task task = NewAddTask (ind_gen);
+			if (isr.ToAdd.Count > 0 || isr.ToRemove.Count > 0) {
+				Log.Debug ("IndexingService: Adding {0} indexables, removing {1} indexables.", isr.ToAdd.Count, isr.ToRemove.Count);
+
+				IndexableGenerator ind_gen;
+				ind_gen = new IndexableGenerator (isr.ToAdd, isr.ToRemove, this);
+				Scheduler.Task task = backend.NewAddTask (ind_gen);
 				task.Priority = Scheduler.Priority.Immediate;
 				ThisScheduler.Add (task);
 			}
@@ -318,6 +393,23 @@ namespace Beagle.Daemon.IndexingServiceQueryable {
 			return new EmptyResponse ();
 		}
 
+		protected override QueryPart QueryPartHook (QueryPart part)
+		{
+			if (part is QueryPart_Property) {
+				QueryPart_Property prop_part = (QueryPart_Property) part;
+				if (prop_part.Key == "inuri") { // special case
+					QueryPart_Property new_part = new QueryPart_Property ();
+					new_part.Logic = prop_part.Logic;
+					new_part.Key = "beagle:inuri";
+					new_part.Type = PropertyType.Text;
+					new_part.Value = prop_part.Value;
+
+					return new_part;
+				}
+			}
+
+			return part;
+		}
 	}
 
 }
