@@ -38,6 +38,7 @@ namespace Search {
 		private Search.Panes panes;
 		private Search.Tray.TrayIcon tray;
 
+		private Search.Pages.IndexInfo indexinfo;
 		private Search.Pages.QuickTips quicktips;
 		private Search.Pages.RootUser rootuser;
 		private Search.Pages.StartDaemon startdaemon;
@@ -56,6 +57,7 @@ namespace Search {
 		private XKeybinder keybinder = new XKeybinder ();
 
 		public static bool IconEnabled = false;
+		private static bool search_docs = false;
 
 		public static void Main (string [] args)
 		{
@@ -97,8 +99,8 @@ namespace Search {
 					IconEnabled = true;
 					break;
 
-				case "--autostarted":
-					// FIXME: This option is deprecated and will be removed in a future release.
+				case "--search-docs":
+					search_docs = true;
 					break;
 
 				// Ignore session management
@@ -133,6 +135,7 @@ namespace Search {
 				"Usage: beagle-search [OPTIONS] [<query string>]\n\n" +
 				"Options:\n" +
 				"  --icon\t\t\tAdd an icon to the notification area rather than opening a search window.\n" +
+				"  --search-docs\t\t\tAlso search the system-wide documentation index.\n" +
 				"  --help\t\t\tPrint this usage message.\n" +
 				"  --version\t\t\tPrint version information.\n";
 
@@ -158,6 +161,7 @@ namespace Search {
 			uim.SortChanged += OnSortChanged;
 			uim.ToggleDetails += OnToggleDetails;
 			uim.ShowQuickTips += OnShowQuickTips;
+			uim.ShowIndexInfo += OnShowIndexInfo;
 			vbox.PackStart (uim.MenuBar, false, false, 0);
 
 			HBox hbox = new HBox (false, 6);
@@ -218,6 +222,10 @@ namespace Search {
 			quicktips.Show ();
 			pages.Add (quicktips);
 
+			indexinfo = new Pages.IndexInfo ();
+			indexinfo.Show ();
+			pages.Add (indexinfo);
+
 			rootuser = new Pages.RootUser ();
 			rootuser.Show ();
 			pages.Add (rootuser);
@@ -309,6 +317,16 @@ namespace Search {
 			}
 		}
 
+		private void DetachQuery ()
+		{
+			if (current_query != null) {
+				TotalMatches = -1;
+				current_query.HitsAddedEvent -= OnHitsAdded;
+				current_query.HitsSubtractedEvent -= OnHitsSubtracted;
+				current_query.Close ();
+			}
+		}
+
 		// Whether we should grab focus from the text entry
 		private bool grab_focus;
 
@@ -340,12 +358,8 @@ namespace Search {
 			this.grab_focus = grab_focus;
 
 			try {
-				if (current_query != null) {
-					TotalMatches = -1;
-					current_query.HitsAddedEvent -= OnHitsAdded;
-					current_query.HitsSubtractedEvent -= OnHitsSubtracted;
-					current_query.Close ();
-				}
+				// Clean up our previous query, if any exists.
+				DetachQuery ();
 
 				TotalMatches = 0;
 
@@ -353,12 +367,14 @@ namespace Search {
 				current_query.AddDomain (QueryDomain.Neighborhood);
 
 				// Don't search documentation by default
-				QueryPart_Property part = new QueryPart_Property ();
-				part.Logic = QueryPartLogic.Prohibited;
-				part.Type = PropertyType.Keyword;
-				part.Key = "beagle:Source";
-				part.Value = "documentation";
-				current_query.AddPart (part);
+				if (! search_docs) {
+					QueryPart_Property part = new QueryPart_Property ();
+					part.Logic = QueryPartLogic.Prohibited;
+					part.Type = PropertyType.Keyword;
+					part.Key = "beagle:Source";
+					part.Value = "documentation";
+					current_query.AddPart (part);
+				}
 
 				current_query.AddText (query);
 				current_query.HitsAddedEvent += OnHitsAdded;
@@ -444,7 +460,6 @@ namespace Search {
 			}
 		}
 		
-
 		private void OnSortChanged (Search.SortType newSort)
 		{
 			view.Sort = sort = newSort;
@@ -461,32 +476,35 @@ namespace Search {
 
 		private void OnShowQuickTips ()
 		{
-			if (current_query != null) {
-				TotalMatches = -1;
-				current_query.HitsAddedEvent -= OnHitsAdded;
-				current_query.HitsSubtractedEvent -= OnHitsSubtracted;
-				current_query.Close ();
-				current_query = null;
-			}
-
+			DetachQuery ();
 			pages.CurrentPage = pages.PageNum (quicktips);
 		}
-
+		
+		private void OnShowIndexInfo ()
+		{
+			DetachQuery ();
+			
+			if (! indexinfo.Refresh ())
+				pages.CurrentPage = pages.PageNum (startdaemon);
+			else
+				pages.CurrentPage = pages.PageNum (indexinfo);
+		}
+		
 		private void OnDomainChanged (QueryDomain domain, bool active)
 		{
 			if (current_query == null)
 				return;
-
+			
 			// FIXME: Most likely refire the query.
 			// Also keep the setting, so it can be used for future queries
 			// in this running instance.
-
+			
 			if (active)
 				current_query.AddDomain (domain);
 			else
 				current_query.RemoveDomain (domain);
 		}
-
+		
 		private void ShowInformation (Tiles.Tile tile)
 		{
 			if (tile != null) {
